package com.ruoyi.generator.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.generator.domain.Community.*;
import com.ruoyi.generator.mapper.community.CommunityArticleCommentMapper;
import com.ruoyi.generator.mapper.community.CommunityClassMapper;
import com.ruoyi.generator.mapper.community.CommunityCommentLikeMapper;
import com.ruoyi.generator.service.ICommunityArticleService;
import com.ruoyi.generator.vo.CommunityArticleVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.text.ParseException;
import java.util.List;
import java.util.Objects;

/**
 * 社区文章管理
 *
 * @author fangzhen
 */
@Api(tags = "社区文章管理")
@RestController
@RequestMapping("/community/article")
public class CommunityArticleController extends BaseController {
    @Autowired
    private ICommunityArticleService communityArticleService;

    @Autowired
    private CommunityClassMapper communityClassMapper;

    @Autowired
    private CommunityArticleCommentMapper communityArticleCommentMapper;

    @Autowired
    private CommunityCommentLikeMapper communityCommentLikeMapper;


    /**
     * 获取文章列表信息
     */
    @ApiOperation("获取文章列表")
    @GetMapping("/list")
    //@Anonymous
    public TableDataInfo communityList(CommunityArticle communityArticle) {
        startPage();
        List<CommunityArticleVo> list = communityArticleService.selectCommunityArticleList(communityArticle);
        return getDataTable(list);
    }

    /**
     * 发布文章
     */
    @ApiOperation("发布文章")
    @PostMapping("/article")
    //@Anonymous
    public AjaxResult article(@RequestBody CommunityArticle communityArticle) {
        communityArticleService.insertCommunityArticle(communityArticle);
        return AjaxResult.success("文章发布成功!");
    }


    @GetMapping("/class")
    @ApiOperation("获取文章分类列表")
    //@Anonymous
    public AjaxResult getClassList() {
        List<CommunityClass> communityClasses = communityClassMapper.selectList(new QueryWrapper<CommunityClass>());
        return AjaxResult.success(communityClasses);
    }

    @ApiOperation("文章评价")
    @PostMapping("/comment")
    public AjaxResult comment(@RequestBody CommunityArticleComment communityArticleComment) throws ParseException {
        if (null == communityArticleComment.getArticleId()) {
            return AjaxResult.error("文章不存在或异常!");
        }

        if (null == communityArticleComment.getUserId()) {
            return AjaxResult.error("用户不存在或异常!");
        }

        //新增评论
        communityArticleService.sendComment(communityArticleComment);

        return AjaxResult.success();
    }

    /**
     * 文章点赞
     */
    @ApiOperation("文章点赞")
    @PostMapping("/like")
    //@Anonymous
    public AjaxResult like(@RequestBody CommunityLike communityLike) {
        if (Objects.isNull(communityLike.getArticleId()) || Objects.isNull(communityLike.getUserId())) {
            return AjaxResult.error("文章或用户信息异常!");
        }
        boolean isSuccess = true;
        String message = "";
        CommunityLike like = communityArticleService.selectCommunityLikeById(communityLike);
        if (Objects.isNull(like)) {
            //新增点赞
            communityLike.setCreateBy(String.valueOf(SecurityUtils.getLoginUser().getUserId()));
            communityLike.setUpdateBy(String.valueOf(SecurityUtils.getLoginUser().getUserId()));
            communityLike.setCreateTime(DateUtils.parseDate(DateUtils.getTime()));
            communityLike.setUpdateTime(DateUtils.parseDate(DateUtils.getTime()));
            isSuccess = communityArticleService.insertCommunityLikeById(communityLike);
            message = "感谢点赞";
        } else {
            //删除点赞
            isSuccess = communityArticleService.deleteCommunityLikeById(like);
            message = "取消点赞";
        }

        if (!isSuccess) {
            message = "操作失败!";
            return AjaxResult.error(message);
        }
        return AjaxResult.success(message);
    }


    /**
     * 获取文章列表信息
     */
    @ApiOperation("文章收藏")
    @PostMapping("/collect")
    //@Anonymous
    public AjaxResult collect(@RequestBody CommunityArticleCollect communityArticleCollect) {
        if (Objects.isNull(communityArticleCollect.getArticleId()) || Objects.isNull(communityArticleCollect.getUserId())) {
            return AjaxResult.error("文章或用户信息异常!");
        }
        boolean isSuccess = true;
        String message = "";
        CommunityArticleCollect collect = communityArticleService.selectCommunityCollectById(communityArticleCollect);
        if (Objects.isNull(collect)) {
            //新增收藏
            isSuccess = communityArticleService.insertCommunityCollectById(communityArticleCollect);
            message = "收藏成功";
        } else {
            //删除收藏
            isSuccess = communityArticleService.deleteCommunityCollectById(collect);
            message = "取消收藏";
        }

        if (!isSuccess) {
            message = "操作失败!";
            return AjaxResult.error(message);
        }
        return AjaxResult.success(message);
    }


    /**
     * 评论点赞
     */
    @ApiOperation("评论点赞")
    @PostMapping("/commentLike")
    //@Anonymous
    public AjaxResult commentLike(@RequestBody CommunityCommentLike commentLike) {
        if (Objects.isNull(commentLike.getCommentId())) {
            return AjaxResult.error("评论或用户信息异常!");
        }
        Long userId = SecurityUtils.getLoginUser().getUserId();
        boolean isSuccess = true;
        String message = "";
        //查询该评论是否已点赞
        List<CommunityCommentLike> commentLikes = communityCommentLikeMapper
                .selectList(new QueryWrapper<CommunityCommentLike>()
                        .eq("comment_id", commentLike.getCommentId())
                        .eq("user_id", userId));

        if (commentLikes.isEmpty()) {
            //新增点赞
            commentLike.setCreateBy(userId);
            commentLike.setUserId(userId);
            commentLike.setUpdateBy(userId);
            commentLike.setCreateTime(DateUtils.parseDate(DateUtils.getTime()));
            commentLike.setUpdateTime(DateUtils.parseDate(DateUtils.getTime()));
            communityCommentLikeMapper.insert(commentLike);
            message = "感谢点赞";
        } else {
            //删除点赞
            communityCommentLikeMapper.deleteById(commentLike);
            message = "取消点赞";
        }

        if (!isSuccess) {
            message = "操作失败!";
            return AjaxResult.error(message);
        }
        return AjaxResult.success(message);
    }
}