package com.ruoyi.generator.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.generator.domain.Community.*;
import com.ruoyi.generator.mapper.community.*;
import com.ruoyi.generator.vo.CommunityArticleCommentVo;
import com.ruoyi.generator.vo.CommunityArticleVo;
import com.ruoyi.system.mapper.SysUserMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * 业务 服务层实现
 *
 * @author ruoyi
 */
@Service
public class CommunityArticleServiceImpl extends ServiceImpl<CommunityArticleMapper, CommunityArticle> implements ICommunityArticleService {
    private static final Logger log = LoggerFactory.getLogger(CommunityArticleServiceImpl.class);

    @Autowired
    private CommunityArticleMapper communityArticleMapper;

    @Autowired
    private SysUserMapper sysUserMapper;

    @Autowired
    private CommunityLikeMapper communityLikeMapper;

    @Autowired
    private CommunityCollectMapper communityCollectMapper;

    @Autowired
    private CommunityArticleCommentMapper communityArticleCommentMapper;

    @Autowired
    private CommunityCommentLikeMapper communityCommentLikeMapper;

    @Autowired
    private CommunityArticleTagMapper articleTagMapper;

    @Autowired
    private CommunityArticleTagMapper communityArticleTagMapper;

    @Autowired
    private CommunityTagMapper communityTagMapper;

    @Autowired
    private CommunityArticleImagesMapper communityArticleImagesMapper;

    /**
     * 查询文章列表
     *
     * @param communityArticle 文章信息
     * @return 文章信息集合
     */
    @Override
    public List<CommunityArticleVo> selectCommunityArticleList(CommunityArticle communityArticle) {
        List<CommunityArticleVo> communityArticleVos = communityArticleMapper.selectCommunityArticleList(communityArticle);
        Long userId = SecurityUtils.getUserId();
        for (CommunityArticleVo articleVo : communityArticleVos) {
            //设置文章的点赞数量
            articleVo.setLikeCount(communityLikeMapper
                    .selectList(new QueryWrapper<CommunityLike>()
                            .eq("article_id", articleVo.getId()))
                    .size());

            //文章下的评论
            List<CommunityArticleCommentVo> comments = articleVo.getComments();
            for (CommunityArticleCommentVo communityArticleCommentVo : comments) {
                //获取评论的用户信息
                SysUser sysUser = sysUserMapper.selectUserById(communityArticleCommentVo.getUserId());
                communityArticleCommentVo.setUsername(sysUser.getUserName());
                communityArticleCommentVo.setAvatar(sysUser.getAvatar());
                //当前登录用户是否已点赞
                List<CommunityCommentLike> commentLikes = communityCommentLikeMapper
                        .selectList(new QueryWrapper<CommunityCommentLike>()
                                .eq("comment_id", communityArticleCommentVo.getId()).eq("user_id", userId));
                communityArticleCommentVo.setCommentLike(!commentLikes.isEmpty());

                //该评论的点赞数量
                communityArticleCommentVo.setCommentLikeCount(communityCommentLikeMapper
                        .selectList(new QueryWrapper<CommunityCommentLike>()
                                .eq("comment_id", communityArticleCommentVo.getId()))
                        .size());
            }

            //判断是否已收藏
            CommunityArticleCollect collect = communityCollectMapper
                    .selectOne(new QueryWrapper<CommunityArticleCollect>()
                            .eq("user_id", userId)
                            .eq("article_id", articleVo.getId()));
            articleVo.setCollect(!Objects.isNull(collect));

            //判断是否已点赞
            CommunityLike like = communityLikeMapper
                    .selectOne(new QueryWrapper<CommunityLike>()
                            .eq("user_id", userId)
                            .eq("article_id", articleVo.getId()));
            articleVo.setLike(!Objects.isNull(like));

            //获取文章标签
            List<CommunityArticleTag> articleTags = communityArticleTagMapper
                    .selectList(new QueryWrapper<CommunityArticleTag>()
                            .eq("article_id", articleVo.getId()));
            List<Long> tagIds = new ArrayList<>();
            for (CommunityArticleTag articleTag : articleTags) {
                tagIds.add(articleTag.getTagId());
            }

            if (!tagIds.isEmpty()) {
                List<CommunityTag> communityTags = communityTagMapper.selectBatchIds(tagIds);
                articleVo.setTags(communityTags);
            }

        }
        return communityArticleVos;
    }


    /**
     * 发布文章
     *
     * @param communityArticle 文章信息
     */
    @Override
    public void insertCommunityArticle(CommunityArticle communityArticle) {
        Long userId = SecurityUtils.getLoginUser().getUserId();
        //插入文章信息
        communityArticle.setUserId(userId);
        communityArticle.setCreateBy(userId);
        communityArticle.setUpdateBy(userId);
        communityArticle.setUpdateTime(DateUtils.parseDate(DateUtils.getTime()));
        communityArticle.setCreateTime(DateUtils.parseDate(DateUtils.getTime()));
        communityArticleMapper.insert(communityArticle);
        //插入标签日志
        List<CommunityTag> tags = communityArticle.getTags();
        CommunityArticleTag tagLog = null;
        for (CommunityTag tag : tags) {
            tagLog = new CommunityArticleTag();
            tagLog.setArticleId(communityArticle.getId());
            tagLog.setTagId(tag.getId());
            tagLog.setUpdateBy(userId);
            tagLog.setUpdateTime(DateUtils.parseDate(DateUtils.getTime()));
            tagLog.setCreateTime(DateUtils.parseDate(DateUtils.getTime()));
            tagLog.setCreateBy(userId);
            articleTagMapper.insert(tagLog);
        }

        //插入图片日志
        List<String> images = communityArticle.getImages();
        CommunityArticleImages articleImages = null;
        for (String image : images) {
            articleImages = new CommunityArticleImages();
            articleImages.setArticleId(communityArticle.getId());
            articleImages.setImageUrl(image);
            articleImages.setUpdateBy(userId);
            articleImages.setUpdateTime(DateUtils.parseDate(DateUtils.getTime()));
            articleImages.setCreateTime(DateUtils.parseDate(DateUtils.getTime()));
            articleImages.setCreateBy(userId);
            communityArticleImagesMapper.insert(articleImages);
        }
    }


    /**
     * 发送评论
     *
     * @param communityArticleComment 评论信息
     */
    @Override
    public void sendComment(CommunityArticleComment communityArticleComment) throws ParseException {
        communityArticleComment.setId(null);
        communityArticleComment.setCreateBy(String.valueOf(SecurityUtils.getLoginUser().getUserId()));
        communityArticleComment.setUpdateBy(String.valueOf(SecurityUtils.getLoginUser().getUserId()));
        communityArticleComment.setCreateTime(DateUtils.parseDate(DateUtils.getTime()));
        communityArticleComment.setUpdateTime(DateUtils.parseDate(DateUtils.getTime()));
        communityArticleCommentMapper.insert(communityArticleComment);
    }


    /**
     * 查询该文章对应的用户是否已经点赞
     *
     * @param communityLike 点赞信息
     * @return 点赞信息
     */
    @Override
    public CommunityLike selectCommunityLikeById(CommunityLike communityLike) {
        QueryWrapper<CommunityLike> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("article_id", communityLike.getArticleId()).eq("user_id", communityLike.getUserId());
        return communityLikeMapper.selectOne(queryWrapper);
    }


    /**
     * 新增点赞
     *
     * @param communityLike 点赞信息
     * @return 点赞是否成功
     */
    @Override
    public boolean insertCommunityLikeById(CommunityLike communityLike) {
        return communityLikeMapper.insert(communityLike) > 0;
    }


    /**
     * 删除点赞
     *
     * @param communityLike 点赞信息
     * @return 删除是否成功
     */
    @Override
    public boolean deleteCommunityLikeById(CommunityLike communityLike) {
        return communityLikeMapper.deleteById(communityLike.getId()) > 0;
    }


    /**
     * 查询该文章对应的用户是否已经收藏
     *
     * @param collect 收藏信息
     * @return 收藏信息
     */
    @Override
    public CommunityArticleCollect selectCommunityCollectById(CommunityArticleCollect collect) {
        return communityCollectMapper
                .selectOne(new QueryWrapper<CommunityArticleCollect>()
                        .eq("article_id", collect.getArticleId())
                        .eq("user_id", SecurityUtils.getUserId()));
    }

    /**
     * 新增收藏
     *
     * @param collect 收藏信息
     * @return 收藏是否成功
     */
    @Override
    public boolean insertCommunityCollectById(CommunityArticleCollect collect) {
        return communityCollectMapper.insert(collect) > 0;
    }

    /**
     * 删除收藏
     *
     * @param collect 收藏信息
     * @return 删除是否成功
     */
    @Override
    public boolean deleteCommunityCollectById(CommunityArticleCollect collect) {
        return communityCollectMapper.deleteById(collect) > 0;
    }
}