package com.ruoyi.generator.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.ruoyi.generator.domain.Community.CommunityArticle;
import com.ruoyi.generator.domain.Community.CommunityArticleCollect;
import com.ruoyi.generator.domain.Community.CommunityArticleComment;
import com.ruoyi.generator.domain.Community.CommunityLike;
import com.ruoyi.generator.vo.CommunityArticleVo;

import java.text.ParseException;
import java.util.List;

/**
 * 业务 服务层
 *
 * @author ruoyi
 */
public interface ICommunityArticleService extends IService<CommunityArticle> {
    /**
     * 查询文章列表
     *
     * @param communityArticle 文章信息
     * @return 文章信息集合
     */
    List<CommunityArticleVo> selectCommunityArticleList(CommunityArticle communityArticle);

    /**
     * 发布文章
     *
     * @param communityArticle 文章信息
     */
    void insertCommunityArticle(CommunityArticle communityArticle);

    /**
     * 发送评论
     *
     * @param communityArticleComment 评论信息
     */
    void sendComment(CommunityArticleComment communityArticleComment) throws ParseException;

    /**
     * 查询该文章对应的用户是否已经点赞
     *
     * @param communityLike 点赞信息
     * @return 点赞信息
     */
    CommunityLike selectCommunityLikeById(CommunityLike communityLike);

    /**
     * 新增点赞
     *
     * @param communityLike 点赞信息
     * @return 点赞是否成功
     */
    boolean insertCommunityLikeById(CommunityLike communityLike);

    /**
     * 删除点赞
     *
     * @param communityLike 点赞信息
     * @return 删除是否成功
     */
    boolean deleteCommunityLikeById(CommunityLike communityLike);

    /**
     * 查询该文章对应的用户是否已经收藏
     *
     * @param collect 收藏信息
     * @return 收藏信息
     */
    CommunityArticleCollect selectCommunityCollectById(CommunityArticleCollect collect);

    /**
     * 新增收藏
     *
     * @param collect 收藏信息
     * @return 收藏是否成功
     */
    boolean insertCommunityCollectById(CommunityArticleCollect collect);

    /**
     * 删除收藏
     *
     * @param collect 收藏信息
     * @return 删除是否成功
     */
    boolean deleteCommunityCollectById(CommunityArticleCollect collect);
}
